import sys
import csv
import json
import types
import warnings
import time
import urllib.request as urllib2
from twitter import Twitter, OAuth, TwitterHTTPError, TwitterStream
import proxySetup

# Variables that contains the user credentials to access Twitter API 
ACCESS_TOKEN = '<your access token here>'
ACCESS_SECRET = '<your access secret here>'
CONSUMER_KEY = '<your consumer key here>'
CONSUMER_SECRET = '<your consumer secret here>'

def authenticate():
    oauth = OAuth(ACCESS_TOKEN, ACCESS_SECRET, CONSUMER_KEY, CONSUMER_SECRET)
    return oauth

def storeTweetsCsv(filepath, tweets):
    with open(filepath, 'w', newline='', encoding="utf-16") as csvfile:
        writer = csv.writer(csvfile, delimiter=',', escapechar="\\",
                                quotechar='"', quoting=csv.QUOTE_ALL)
        
        #writer.writerow(['user_screen_name', 'user_name', 'created_at', 'text'])
        
        for tweet in tweets:
            tmp = list([tweet['user']['screen_name'], tweet['user']['name'], tweet['created_at'], tweet['text']])
            writer.writerow(tmp)
            
        csvfile.close()
        
    pass

def filterTweets(tweets):
    filteredTweets = {'root':[]}
    
    for tweet in tweets:
        if 'text' in tweet and 'user' in tweet:
            filteredTweet = {}
            user = {}
            user['name'] = tweet['user']['name']
            user['created_at'] = tweet['user']['created_at']
            user['friends_count'] = tweet['user']['friends_count']
            user['screen_name'] = tweet['user']['screen_name']
            user['lang'] = tweet['user']['lang']
            user['followers_count'] = tweet['user']['followers_count']
            user['id'] = tweet['user']['id']
            filteredTweet['user'] = user
            filteredTweet['source'] = tweet['source']
            filteredTweet['timestamp_ms'] = tweet['timestamp_ms']
            filteredTweet['in_reply_to_screen_name'] = tweet['in_reply_to_screen_name']
            filteredTweet['created_at'] = tweet['created_at']
            filteredTweet['retweeted'] = tweet['retweeted']
            filteredTweet['id_str'] = tweet['id_str']
            filteredTweet['truncated'] = tweet['truncated']
            filteredTweet['retweet_count'] = tweet['retweet_count']
            filteredTweet['coordinates'] = tweet['coordinates']
            filteredTweet['text'] = tweet['text']
            filteredTweet['lang'] = tweet['lang']
            filteredTweet['id'] = tweet['id']
            filteredTweet['geo'] = tweet['geo']
            if 'retweeted_status' in tweet:
                retweededStatus = {}
                retweededStatus['retweet_count'] = tweet['retweeted_status']['retweet_count']
                filteredTweet['retweeted_status'] = retweededStatus
            filteredTweet['geo'] = tweet['geo']

            filteredTweets['root'].append(filteredTweet)
    
    return filteredTweets['root']

def streamSampleTweets(noOfTweets, oauth, language=None):
    # Initiate the connection to Twitter Streaming API
    twitter_stream = TwitterStream(auth=oauth)
    
    # Get a sample of the public data following through Twitter
    iterator = twitter_stream.statuses.sample()
    
    tweets = []
    
    # Print each tweet in the stream to the screen 
    # Here we set it to stop after getting 1000 tweets. 
    # You don't have to set it to stop, but can continue running 
    # the Twitter API to collect data for days or even longer. 
    tweet_count = noOfTweets
    for tweet in iterator:
        #tweet_count -= 1
        # Twitter Python Tool wraps the data returned by Twitter 
        # as a TwitterDictResponse object.
        if 'text' in tweet and 'user' in tweet:
            if language != None:
                if ('lang' in tweet) and (tweet['lang'] in language):
                    tweets.append(tweet)
                    tweet_count -= 1
            else:
                tweets.append(tweet)
                tweet_count -= 1

        if tweet_count <= 0:
            break
    return tweets

def streamTweets(track, noOfTweets, oauth, lang='en'):
    # Initiate the connection to Twitter Streaming API
    twitter_stream = TwitterStream(auth=oauth)
    
    # Get a sample of the public data following through Twitter
    #iterator = twitter_stream.statuses.sample()
    iterator = twitter_stream.statuses.filter(track=track, language=lang)
    
    tweets = []
    
    # Print each tweet in the stream to the screen 
    # Here we set it to stop after getting 1000 tweets. 
    # You don't have to set it to stop, but can continue running 
    # the Twitter API to collect data for days or even longer. 
    tweet_count = noOfTweets
    for tweet in iterator:
        tweet_count -= 1
        # Twitter Python Tool wraps the data returned by Twitter 
        # as a TwitterDictResponse object.
        if 'text' in tweet and 'user' in tweet:
            tweets.append(tweet)
            
        if tweet_count <= 0:
            break
    return tweets

def main():
    start_time = time.time()
    
    proxySetup.setupProxy()
    
    streamSamples = True
    
    print('Authenticating...')
    oauth = authenticate()
    
    tweetCount = 1000

    if not streamSamples:
        track = 'usa'
        language = 'en'
    
        trackStr = '' if track is None else ' (' + track + ')'
        print('Fetching ' + str(tweetCount) + ' tweets' + trackStr + '...')
        tweets = streamTweets(track, tweetCount, oauth, language)
    else:
        track = 'SAMPLEDATA'
        language = ['en']
        
        print('Fetching ' + str(tweetCount) + ' tweets (SAMPLE DATA)...')
        tweets = streamSampleTweets(tweetCount, oauth, language)
    
    print('Filtering tweets...')
    filteredTweets = filterTweets(tweets)
    
    print('Storing tweets...')
    localtime = time.localtime()
    timeString = time.strftime("%Y%m%d_%H%M%S", localtime)
    filepath = 'storedTweets/' + timeString + '-tweets-' + track + '-' + str(tweetCount) + '.csv'
    #storeTweetsJson(filepath, filteredTweets)
    storeTweetsCsv(filepath, filteredTweets)
    print('  => Stored tweets to "' + filepath + '".')
    
    print('DONE.')
    print('total time in s: %.4f' % (time.time() - start_time))
    
    pass


if __name__ == '__main__':
    warnings.simplefilter('ignore') # ignore all warnings
    #warnings.filterwarnings('ignore', category=UserWarning, append=False)
    #warnings.filterwarnings('ignore', category=DeprecationWarning, append=False)
    #warnings.filterwarnings('ignore', category=FutureWarning, append=False)
    
    sys.exit(main())